package com.ibm.vap.cciadapter;

/*
 * Source File Name : CCIRequester.java
 * Licensed Materials - Property of IBM
 * 5655-F37
 * (C) Copyright IBM Corp. 1983, 2002. All Rights Reserved.
 */
import com.ibm.vap.generic.*;
import javax.resource.cci.*;

/**
 * Class used to adapt the J2EE connectors to the Visualage Pacbase Runtime.
 * This class is a subclass of the com.ibm.vap.generic.Requester.
 * @since 2.5v11
 */
public class CCIRequester extends Requester {
	private final static String copyright = "Licensed Materials - Property of IBM\n5655-F37\n(C) Copyright IBM Corp. 1983, 2002. All Rights Reserved.\nUS Government Users Restricted Rights - Use, duplication or disclosure restricted by GSA ADP\nSchedule Contract with IBM Corp.";
	protected Connection connection;
	protected Interaction interaction;
	protected CCIAdapter adapter;
	protected VapRecord outBuffer;
protected CCIRequester(CCIAdapter cciAdapter, Connection c) throws CommunicationError {
	adapter = cciAdapter;
	connection = c;
}
protected VapRecord createVapRecord() {
	return new VapRecord(this.adapter);
}
protected VapRecord createVapRecord(int length) {
	return new VapRecord(length, adapter);
}
protected Connection getConnection() {
	return connection;
}
/**
 * Gets the maximum message length.
 * Return the maximum message length it requires.
 * The VisualAge Pacbase runtime garanties that no buffer passed to the function "send" will
 * be longer than this maximum.
 * If this function returns a value equals to or lower than 0, no maximum message
 * length is considered by the VisualAge Pacbase Runtime.
 */
public int getMaxMessageLength() throws CommunicationError { 
	return adapter.getCommareaLength(); 
}
/**
 * Waits for a response associated to the last request.
 * This function must wait until at least one character has been received from the
 * VisualAge Pacbase communication monitor at the server side.
 * The received characters must be stored in the given char array, starting at the given offset and
 * stopping when either "length" characters have been copied or no more character is available for
 * reception.
 * The function must return the total number of characters currently received.
 * When this method returns the value "nbReturn", two cases can be considered :
 * - "nbReturn <= length" : only "nbReturn" characters have been currently received and are stored in the
 *  char array. A subsequent call to "receive" will likely block.
 * - "nbReturn > length" : more than "length" characters have been received, but only the first "length"
 *  have been stored in the char array. Subsequent calls to "receive" will not block until at least all the
 *  remaining characters (nbReturn-length) have been returned.
 * Calling "receive" with a length of 0 is a good way to wait for characters without getting any of them (thus,
 * without having to allocate a char array). Then "receive" can be called a second time to get all the currently
 * received characters (passing the value returned by the first call as "length" parameter).
 * Since this method must wait until characters are received, it should never return 0.
 * The only case where the returned value can be 0 is when a receive timeout has expired.
 * @exception CommunicationError is thrown if an error condition is encountered.
 */
public synchronized int receive( char[] message, int offset, int length ) throws CommunicationError
{
	try	{
		// Transfer the outBuffer to the current message parameter
		outBuffer.getMessage(message,offset,length);
		return length;
	}
	catch ( Exception e ) { 
		throw new CommunicationError( "Fail while retrieving the reply message due to : "+ e.getMessage() );
	}
}
/**
 * Checks for a response associated to the last request, without waiting.
 * The method returns the number of characters currently received.
 * When this method returns 0, it means that a subsequent call to "receive" will likely block.
 * Else, it means that a subsequent call to receive will not block.
 */
public int receiveCheck() throws CommunicationError {
	try	{
		// Transfer the outBuffer to the current message parameter
		return outBuffer.getBufferLength();
	}
	catch ( Exception e ) { 
		throw new CommunicationError( "Fail while retrieving the reply message length due to : "+ e.getMessage() );
	}
}
/**
 * Releases the requester.
 * Once released, the methods of the requester have an unpredictable result and thus should not be called.
 */
public void release() {
	try {
		if (interaction != null)
			interaction.close();
		interaction = null;
		outBuffer = null;
	} catch (javax.resource.ResourceException rse) {
		//We do not need to signal this error, it must still work properly
	}
	// Ask the adapter to release the requester : maybe the prefered one !
	adapter.releaseRequester(this);
}
/**
 * Sends a message to the server.
 * The given message must be passed to the Visualage Pacbase communication monitor at the server side.
 * @param message to be sent to the server
 * @param offset defines the starting charactere where to start sending the message
 * @param length of the message to be sent
 * @return the number of bytes effectively sent
 * @exception CommunicationError if the interaction can't be instantiate or the execute method fails.
 */
public synchronized int send(char[] message, int offset, int length)
	throws CommunicationError {
	// Instantiate a new interaction instance if not already done
	if (interaction == null) {
		try {
			interaction = connection.createInteraction();
		} catch (javax.resource.ResourceException rse) {
			throw new CommunicationError(
				"The following error occured trying to make a connection : "
					+ rse.getErrorCode());
		}
	}
	try {
		// Transfer the message to send in the input VapRecord instance used by the connector
		VapRecord inBuffer = createVapRecord();
		inBuffer.setMessage(message, offset, length);

		outBuffer = createVapRecord(getMaxMessageLength());

		interaction.execute(adapter.getInteractionSpec(), inBuffer, outBuffer);
	} catch (javax.resource.ResourceException rse) {
		throw new CommunicationError(
			"Fail to communicate with the J2EE Connectors due to : " + rse.getErrorCode(), rse);
	} catch (java.io.IOException e) {
		throw new CommunicationError(
			"Fail to communicate with the J2EE Connectors due to : " + e.getMessage(), e);
	}
	return length;
}
}
